# -*- coding: utf-8 -*-


class AccountActivityTypeV1:
    AccountCreated: str = 'account created'
    AccountChanged: str = 'account changed'
    AccountDeleted: str = 'account deleted'
    AccountDropped: str = 'account dropped'
