# -*- coding: utf-8 -*-

__all__ = ['IGroupMembershipsService', 'GroupMembershipsService']

from .IGroupMembershipsService import IGroupMembershipsService
from .GroupMembershipsService import GroupMembershipsService
